#  ==========================================================================
#  floating_camera.rb
#  --------------------------------------------------------------------------
#
#  Module	:	CControl (Module for Controlling the Camera)
#  Author	:	Niall Campbell, 2011-08(-31)
#  Version	:	1.0.0(b1.4) 2011-08(-31)
#
#  --------------------------------------------------------------------------
#  ==========================================================================
#  --------------------------------------------------------------------------
#
#  Description	:	Script for controlling the camera in sketchup. See Users
#					guide for further information
#
# Permission to use this software for any purpose and without fee is hereby granted
# Distribution of this software for commercial purpose is subject to:
#  - the expressed, written consent of the author
#  - the inclusion of the present copyright notice in all copies.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  --------------------------------------------------------------------------
#  ==========================================================================
#  --------------------------------------------------------------------------
#
#  CHANGELOG:
#  ------------------
# b1.4x made >=v2014 compatible TIG 20150606
#  = v 1.0.0(b1.4) (2011-08-31)
#	-Fixed slow framerate issue on collision & eyeheight enabled by removing sane distance
#	 test
#  = v 1.0.0(b1.3) (2011-08-30)
#	-Fixed slow framerate issue by removing ruby:start_operation & ruby:abort_operation
#  = v 1.0.0(b1.2) (2011-08-30)
#  	-Fixed Sketchup 7 collision detection bug
#  = v 1.0.0(b1) (2011-08-27)
#   -Initial public beta release
#
#  --------------------------------------------------------------------------
#  ==========================================================================
require 'sketchup.rb'

module CControl


	class CameraFloatTool
		require(File.join( File.dirname( __FILE__) ,'floating_camera/mixin/load_instructor.rb'))
		include( Mixin::LoadInstructor )

		def initialize
			# store the location of the instructor folder
			# relative to this scripts directory
			instructor_folder = File.join( File.dirname( __FILE__) ,'floating_camera', 'instructor')
			# if on pc, store the location of the temp 
			# instructor folder relative to the system
			# temp directory
			temp_folder       = (RUBY_PLATFORM =~ /darwin/i) ? '' : File.join(ENV["TEMP"], 'floating_camera', 'instructor')
			# load lins setup with these variables
			lins_setup('', instructor_folder, temp_folder.tr("\\","/"))
		end
    
		def getInstructorContentDirectory
			unless @@temp_instructor_folder
				lins_copy_instructor
				lins_return_temp_instructor_folder_name
			end
			lins_return_relative_path
		end #def	

		#quality settings (see readme for descriptions)
		@@q_refresh_rate = 0.04
		@@q_frender_delay_min = 5
		@@q_frender_delay_max = 10
		@@q_render_opts = Hash[
			"ModelTransparency" => false,
			"EdgeType"        => 0,
			"EdgeDisplayMode" => 1,
			"EdgeColorMode"   => 1,
			"DrawSilhouettes" => false,
			"DrawDepthQue"    => false,
			"DrawHidden"      => false,
			"DrawLineEnds"    => false,
			"ExtendLines"     => false,
			"JitterEdges"     => false,
			"DisplayDims"     => false,
			"HideConstructionGeometry" => true,
			"MaterialTransparency" => false,
			"ShowViewName" => false
		]
		@@q_shadow_info = Hash[
			"DisplayOnAllFaces" => false,
			"DisplayShadows" => false
		]
		
		@@q_degrade_opts = Hash[
			"Texture" 			=> false,
			"TransparencySort" 	=> 0,
			"DisplayFog" 		=> false
		]
		
		@@d_settings = Hash[
			"max_rotation"    => 540,
			"max_yrotation"   => 90,
			"max_speed_kh"    => 50,
			"friction_scalar" => 10,
			"infinite_rot"    => true,
			"inf_rotation"    => 180,
			"inf_yrotation"   => 45,
			"collision"  => false,
			"complexcl"  => false,
			"restrictv"  => false,
			"camerahgt"  => false,
			"cameraupr"  => false
		];
		
		
		
		#store the default key ids for movement (avoids modifying the constants ;)
		@@c_key_ids = Hash[	
			VK_UP		=> "forward",
			VK_DOWN		=> "backward",
			VK_LEFT		=> "left",
			VK_RIGHT	=> "right",
			VK_SHIFT	=> "up",
			VK_CONTROL	=> "down",
			VK_HOME		=> "shrink",
			VK_END		=> "grow",
			102			=> "clock",
			100			=> "anti",
			VK_INSERT	=> "collision",
			VK_DELETE	=> "restrictv",
			191			=> "camerahgt",
			222			=> "cameraupr"
		]
		
		@@c_key_names = @@c_key_ids.invert
		
		#active movement array
		@@im_values = Hash[
			"forward"   => false,
			"backward"  => false,
			"left"      => false,
			"right"     => false,
			"up"        => false,
			"down"      => false,
			"shrink"    => false,
			"grow"      => false,
			"clock"     => false,
			"anti"      => false
		]
		
		#shortcut values

		#standard keycodes (same as javascript keycodes)
		@@c_keycode = Hash[
			"F2"		=> 113,		"F3"		=> 114,		"F4"		=> 115,		"F5"		=> 116,		"F6"		=> 117,
			"F7"		=> 118,		"F8"		=> 119,		"F9"		=> 120,		"F11"		=> 122,		"F12"		=> 123,
			"F13"		=> 124,		"`"			=> 192,		"0"			=> 48,		"1"			=> 49,		"2"			=> 50,
			"3"			=> 51,		"4"			=> 52,		"5"			=> 53,		"6"			=> 54,		"7"			=> 55,
			"8"			=> 56,		"9"			=> 57,		"-"			=> 189,		"="			=> 187,		"Backspace"	=> 8,
			"Q"			=> 81,		"W"			=> 87,		"E"			=> 69,		"R"			=> 82,		"T"			=> 84,
			"Y"			=> 89,		"U"			=> 85,		"I"			=> 73,		"O"			=> 79,		"P"			=> 80,
			"["			=> 219,		"]"			=> 221,		"\\"		=> 220,		"A"			=> 65,		"S"			=> 83,
			"D"			=> 68,		"F"			=> 70,		"G"			=> 71,		"H"			=> 72,		"J"			=> 74,
			"K"			=> 75,		"L"			=> 76,		";"			=> 186,		"'"			=> 222,		"Z"			=> 90,
			"X"			=> 88,		"C"			=> 67,		"V"			=> 86,		"B"			=> 66,		"N"			=> 78,
			"M"			=> 77,		","			=> 188,		"."			=> 190,		"/"			=> 191,		"Insert"	=> 45,
			"Home"		=> 36,		"End"		=> 35,		"NumPad/"	=> 111,		"NumPad*"	=> 106,		"NumPad-"	=> 109,
			"NumPad+"	=> 107,		"NumPad."	=> 110,		"NumPad0"	=> 96,		"NumPad1"	=> 97,		"NumPad2"	=> 98,
			"NumPad3"	=> 99,		"NumPad4"	=> 100,		"NumPad5"	=> 101,		"NumPad6"	=> 102,		"NumPad7"	=> 103,
			"NumPad8"	=> 104,		"NumPad9"	=> 105,		"Shift"		=> 16,		"Control"	=> 17
		]
		
		@@c_toggle_keycode = @@c_keycode
		@@c_toggle_keycode["Delete"] = 46
		
		
		@@c_key = @@c_keycode.invert
		
		
		
		def getUsedShortcuts
			@@s_shortcuts = Sketchup.get_shortcuts
			@@used_keycodes = Hash.new()
			@@s_shortcuts.each {|shortcut_string|
				shortcut_array = shortcut_string.split("\t")
				shortcut_key = shortcut_array[0]
				keycode = @@c_keycode[shortcut_key]
				keycode = keycode.to_f
				next if shortcut_key == "Delete"
				@@used_keycodes[keycode] = shortcut_array[1] if (!shortcut_key.index('+') && keycode)
			}
		end
		
		@@r_settings_association = Hash[
			"max_rotation"    => "DMROTATION",
			"max_speed_kh"    => "DMSPEEDKH",
			"friction_scalar" => "DFRICTION",
			"inf_rotation"    => "DIROTATION",
			"infinite_rot" => "DINFROTAT",
			"collision"    => "DCOLLISION",
			"restrictv"    => "DRESTRICTV",
			"camerahgt"    => "DCAMERAHGT",
			"cameraupr"    => "DCAMERAUPR"
		];
		
		def loadSettings
			@@r_settings_association.each {|setting_name, default_name|
				@@d_settings[setting_name] = Sketchup.read_default("Floating Camera",default_name,' ')
			}
			@@d_settings["max_yrotation"] = @@d_settings["max_rotation"]/2
			@@d_settings["inf_yrotation"] = @@d_settings["inf_rotation"]/4
		end #def
		
		def openSettingsPanel
			if (!@settdlg || (@settdlg && !@settdlg.visible?))
				@settdlg = UI::WebDialog.new("Motion Settings",false,"CameraFloatToolSettings",280,330,150,150,false)
				#dialog.set_size(280,330)
				plugin_path = File.dirname( __FILE__ )
				settings_location = plugin_path + "/floating_camera/settings.html"
				@settdlg.set_file settings_location
				#update fields with the defaults
				settings_array = Array.new()
				@@r_settings_association.each {|setting_name, default_name|
					settings_array.push(setting_name+":" + Sketchup.read_default("Floating Camera",default_name,' ').to_s)
				}
				control_string = settings_array * "|"
				@settdlg.add_action_callback("ruby_loadsettings") {|dlg, params|
					dlg.execute_script("loadSettings('"+control_string+"')")
				}
				
				@settdlg.add_action_callback("ruby_reloadsettingsdialog") {|dlg, params|
					self.openSettingsPanel
				}
				
				@settdlg.add_action_callback("ruby_savesettings") {|dlg, params|
					settings_valuepairs = params.split("|");
					settings_valuepairs.each {|setting_valuepair|
						setting_valuepair = setting_valuepair.split(":")
						setting_name = setting_valuepair[0]
						setting_value = setting_valuepair[1]
						if (setting_value == 'true' || setting_value == 'false') 
							setting_value = (setting_value == 'true') ? true : false
						else
							setting_value = setting_value.to_f
							setting_value = setting_value.round
						end #
						Sketchup.write_default("Floating Camera",@@r_settings_association[setting_name],setting_value)
						@@d_settings[setting_name] = setting_value
					}
					@@d_settings["max_yrotation"] = @@d_settings["max_rotation"]/2
					@@d_settings["inf_yrotation"] = @@d_settings["inf_rotation"]/1.33
					
					@@d_settings["friction_scalar"] = 0.80 + (@@d_settings["friction_scalar"].quo(100))

					@model_units = self.units
					@max_speed = self.distanceConverter(@@d_settings["max_speed_kh"],'km',@model_units)
					@max_speed = @max_speed.quo(3600)
					@max_speed_in = self.distanceConverter(@max_speed,@model_units,'in')
					if(@@d_settings["infinite_rot"])
						@max_rotation = @@d_settings["inf_rotation"].quo(108000)
						@max_yrotation = @@d_settings["inf_yrotation"].quo(108000)
					else
						@max_rotation = @@d_settings["max_rotation"].quo(108000)
						@max_yrotation = @@d_settings["max_yrotation"].quo(108000)
					end
					
					#work out the relationship of acceleration to friction
					a = -9.92874960
					b =  2.29290294
					c = -1.51307210
					y = c.quo(1+(a*Math.exp(-b*@@d_settings["friction_scalar"])))
					#modify acceleration to respond to the friction applied to it
					@acc_constant = (((@@d_settings["max_speed_kh"]*(20.quo(y)))*11.65) + 34)
					
					@collision_buffer = 10 + ((@max_speed_in.quo(@@q_refresh_rate)).quo(400))

					dlg.close
				}
				
				@settdlg.show
			elsif (@settdlg && @settdlg.visible?)
				@settdlg.bring_to_front
			end
		end #def
		
		
		def openControlsPanel
			if (!@contdlg || (@contdlg && !@contdlg.visible?))
				@contdlg = UI::WebDialog.new("Set Controls",false,"CameraFloatToolControls",280,380,150,150,false)
				@contdlg.set_size(230,373)
				plugin_path = File.dirname( __FILE__ )
				controls_location = plugin_path + "/floating_camera/controls.html"
				self.getUsedShortcuts
				@contdlg.set_file controls_location
				
				#update fields with the defaults
				control_array = Array.new()
				control_array.push(Sketchup.read_default("Floating Camera","KCOLLISION",' '))
				control_array.push(Sketchup.read_default("Floating Camera","KRESTRICTV",' '))
				control_array.push(Sketchup.read_default("Floating Camera","KCAMERAHGT",' '))
				control_array.push(Sketchup.read_default("Floating Camera","KCAMERAUPR",' '))
				control_array.push(Sketchup.read_default("Floating Camera","KUP",' '))
				control_array.push(Sketchup.read_default("Floating Camera","KDOWN",' '))
				control_array.push(Sketchup.read_default("Floating Camera","KCLOCK",  ' '))
				control_array.push(Sketchup.read_default("Floating Camera","KANTI",   ' '))
				control_array.push(Sketchup.read_default("Floating Camera","KSHRINK", ' '))
				control_array.push(Sketchup.read_default("Floating Camera","KGROW",   ' '))
				control_string = control_array * "|"
				@contdlg.add_action_callback("ruby_loadcontrols") {|dlg, params|
					dlg.execute_script("loadControls('"+control_string+"')")
				}
				
				@contdlg.add_action_callback("ruby_reloadcontrolsdialog") {|dlg, params|
					self.openControlsPanel
				}
				
				@contdlg.add_action_callback("ruby_checkshortcuts") {|dlg, params|
					params = params.split("+")
					final = params[2]
					if(final == 'false') 
						keycode, input_id = params[0].to_f, params[1]
						dlg.execute_script("usedKeyCode('"+@@used_keycodes[keycode]+"','"+input_id+"')") if @@used_keycodes[keycode]
					else
						keycode = params[0]
						keycodes = keycode.split("|")
						used_keycodes = Hash.new()
						keycodes.each {|kstring|
							kstring = kstring.split(":")
							kcode = kstring[0].to_f
							used_keycodes[kcode] = @@used_keycodes[kcode] if @@used_keycodes[kcode]
						}
						if used_keycodes.length > 0
							#return an alert to the user
							used_keys, used_assignments = used_keycodes.keys, used_keycodes.values
							used_keys, used_assignments = used_keys * "|", used_assignments * "|"
							dlg.execute_script("errorKeyCodes('"+used_keys+"','"+used_assignments+"')")
						else 
							#store the new keycode assignments
							#keycodes should come back in in order
							if (final != 'check')
								key_names = Array.new(["collision","restrictv","camerahgt","cameraupr","up","down","clock","anti","shrink","grow"])
								i = 0
								keycodes.each{|kstring|
									#remove and update existing key ids
									kstring = kstring.split(":")
									kcode = kstring[0].to_f
									kcode = kcode.round
									@@c_key_ids.delete(@@c_key_names[key_names[i]])
									@@c_key_names[key_names[i]] = kcode
									@@c_key_ids[kcode] = key_names[i]
									#update the defaults & first-run switch
									i=i.next
								}
								#update the defaults & first-run switch
								Sketchup.write_default("Floating Camera","INIT",true)
								Sketchup.write_default("Floating Camera","KCOLLISION",@@c_key_names["collision"])
								Sketchup.write_default("Floating Camera","KRESTRICTV",@@c_key_names["restrictv"])
								Sketchup.write_default("Floating Camera","KCAMERAHGT",@@c_key_names["camerahgt"])
								Sketchup.write_default("Floating Camera","KCAMERAUPR",@@c_key_names["cameraupr"])
								Sketchup.write_default("Floating Camera","KUP",    @@c_key_names["up"])
								Sketchup.write_default("Floating Camera","KDOWN",  @@c_key_names["down"])
								Sketchup.write_default("Floating Camera","KSHRINK",@@c_key_names["shrink"])
								Sketchup.write_default("Floating Camera","KGROW",  @@c_key_names["grow"])
								Sketchup.write_default("Floating Camera","KCLOCK", @@c_key_names["clock"])
								Sketchup.write_default("Floating Camera","KANTI",  @@c_key_names["anti"])
								#close the dialog window
								dlg.close
							end #if
						end #if
					end #if
				}
				@contdlg.show
			elsif (@contdlg && @contdlg.visible?)
				@contdlg.bring_to_front
			end #if
		end #def
		
		def loadControls
			indexes = Hash[
				"KCOLLISION" => "collision",
				"KRESTRICTV" => "restrictv",
				"KCAMERAHGT" => "camerahgt",
				"KCAMERAUPR" => "cameraupr",
				"KUP" => "up",
				"KDOWN" => "down",
				"KSHRINK" => "shrink",
				"KGROW" => "grow",
				"KCLOCK" => "clock",
				"KANTI" => "anti"
			]
			
			indexes.each {|default_id, control_name|
				@@c_key_ids.delete(@@c_key_names[control_name])
				@@c_key_names[control_name] = Sketchup.read_default("Floating Camera",default_id, false)
				@@c_key_ids[@@c_key_names[control_name]] = control_name
			}
		end #def
		
		def checkControls
			indexes = Array.new(["collision","restrictv","camerahgt","cameraupr","up","down","shrink","grow","clock","anti"])
			self.getUsedShortcuts
			used_keycodes = Hash.new()
			indexes.each {|control_name|
				kcode = @@c_key_names[control_name].to_f
				used_keycodes[kcode] = @@used_keycodes[kcode] if @@used_keycodes[kcode]
			}
			if used_keycodes.length > 0
				self.openControlsPanel
			end #if
		end #def
		

		def activate
			# plugin_filepath = __FILE__
			# puts plugin_filepath
			# plugin_path = File.dirname(__FILE__)
			# puts plugin_path
			# root_dir_entries = Dir.entries(plugin_path+"/../../../../../../../")
			# puts root_dir_entries
			$VERBOSE = nil
			@mouse_stored = @animation_start = @animation_active = @contdlg = @settdlg = false
			@mousex = @mousey = 0
			@speed_fov = @speed_look = @speed_move = @speed_roll = @speed_height = @speed_float = 0
			@eye_height = 0
			@p_vector_fov = @p_vector_look = @p_vector_roll = @p_vector_height = @p_vector_float = Geom::Vector3d.new(0,0,0)
			#run the input webdialog
			if Sketchup.read_default("Floating Camera","INIT",false)
				#correct controls with the stored defaults
				self.loadControls
				self.checkControls
			else
				#run the controls webdialog
				self.openControlsPanel
			end #if
			
			self.loadSettings

			@@d_settings["friction_scalar"] = 19 if @@d_settings["friction_scalar"] > 19
			@@d_settings["friction_scalar"] = 0  if @@d_settings["friction_scalar"] < 0
			@@d_settings["friction_scalar"] = 0.80 + (@@d_settings["friction_scalar"].quo(100))
			
			view = Sketchup.active_model.active_view
			camera_normal = view.camera.target - view.camera.eye
			camera_normal.normalize
			camera_cross = camera_normal.cross(Geom::Vector3d.new(0,0,1))
			camera_cross.normalize
			@p_vector_move = camera_normal + camera_cross
			
			#model units
			@model_units = self.units
			@max_speed = self.distanceConverter(@@d_settings["max_speed_kh"],'km',@model_units)
			@max_speed = @max_speed.quo(3600)
			@max_speed_in = self.distanceConverter(@max_speed,@model_units,'in')
			if(@@d_settings["infinite_rot"])
				@max_rotation = @@d_settings["inf_rotation"].quo(108000)
				@max_yrotation = @@d_settings["inf_yrotation"].quo(108000)
			else
				@max_rotation = @@d_settings["max_rotation"].quo(108000)
				@max_yrotation = @@d_settings["max_yrotation"].quo(108000)
			end
			#store the users render settings
			self.storeRenderOptions
			#set some initial render settings for quicker refresh
			self.setRenderOptions(@@q_refresh_rate,@@q_frender_delay_min,@@q_frender_delay_max,@@q_render_opts,@@q_shadow_info)
			
			#create the collision group
			entities = Sketchup.active_model.active_entities
			@collision_group = entities.add_group
			
			#initialise some constant variables for the nextframe loop
			@rad_90 = 90.degrees
			@rad_reduceby2 = 178.degrees/180.degrees
			@rad_1 = 1.degrees
			
			#work out the relationship of acceleration to friction
			a = -9.92874960
			b =  2.29290294
			c = -1.51307210
			y = c.quo(1+(a*Math.exp(-b*@@d_settings["friction_scalar"])))
			#modify acceleration to respond to the friction applied to it
			@acc_constant = (((@@d_settings["max_speed_kh"]*(20.quo(y)))*11.65) + 34)
			
			
			@max_height_vector = Geom::Vector3d.new(0,0,-1)
			@max_height_vector.length = @max_height_vector.length * view.model.bounds.height
			@collision_buffer = 10 + ((@max_speed_in/@@q_refresh_rate)/400)
		end
		
		def deactivate(view)
			if(@animation_active)
				@animation_active = false
				view.dynamic = false
			end #if
			#restore the users render settings
			self.setRenderOptions(@@s_refresh_rate,@@s_frender_delay_min,@@s_frender_delay_max,@@s_render_opts,@@s_shadow_info)
		end
		
		

		
		def onCancel(reason, view)
			if(@animation_active)
				@animation_active = false
				view.dynamic = false
				view.animation = nil
			end #if
			#select the default tool
			Sketchup.active_model.select_tool nil
			#restore the users render settings
			self.setRenderOptions(@@s_refresh_rate,@@s_frender_delay_min,@@s_frender_delay_max,@@s_render_opts,@@s_shadow_info)
		end #def

		def suspend(view)
			if(@animation_active)
				@animation_active = false
				view.dynamic = false
				view.animation = nil
			end #if
			Sketchup.active_model.select_tool nil
			#restore the users render settings
			self.setRenderOptions(@@s_refresh_rate,@@s_frender_delay_min,@@s_frender_delay_max,@@s_render_opts,@@s_shadow_info)
		end
		
		def resume(view)
			#set some initial render settings for quicker refresh
			self.setRenderOptions(@@q_refresh_rate,@@q_frender_delay_min,@@q_frender_delay_max,@@q_render_opts,@@q_shadow_info)
		end
		
		
		def storeRenderOptions
			@@s_render_opts = Hash.new()
			@@s_shadow_info = Hash.new()
			Sketchup.active_model.rendering_options.each_pair do |k,v|
				@@s_render_opts[k] = v
			end #do
			Sketchup.active_model.shadow_info.each_pair do |k,v|
				@@s_shadow_info[k] = v
			end #do
			@@s_degrade_opts = Hash.new()
			@@q_degrade_opts.each_pair do |k,v|
				@@s_degrade_opts[k] = Sketchup.active_model.rendering_options[k]
			end #do
			@@s_refresh_rate      = Sketchup.max_draw_time
			@@s_frender_delay_min = Sketchup.full_detail_render_delay_min
			@@s_frender_delay_max = Sketchup.full_detail_render_delay_max
		end #def
		
		def setRenderOptions(refresh=false,delay_min=false,delay_max=false,render_opts=false,shadow_info=false)
			Sketchup.max_draw_time = refresh if refresh
			Sketchup.full_detail_render_delay_min = delay_min if delay_min
			Sketchup.full_detail_render_delay_max = delay_max if delay_max
			render_opts.each_pair do |k,v|
				Sketchup.active_model.rendering_options[k] = v
			end #do
			if shadow_info
				shadow_info.each_pair do |k,v|
					Sketchup.active_model.shadow_info[k] = v
				end #do
			end #if
		end #def
		
		def units
			x = '120'.to_l.inspect.to_i
			case x
				when  120    then 'in'
				when 1440    then 'ft'
				when 4320    then 'yard'
				when 7603200 then 'mile'
				when    4    then 'mm'
				when   47    then 'cm'
				when 4724    then 'm'
				else 'in'
			end
		end #def
		
		def distanceConverter(distance,unit,result)
			#convert the distance to inches first
			distance_inches = case unit
				when 'in'   then distance.inch
				when 'ft'   then distance.feet
				when 'yard' then distance.yard
				when 'mile' then distance.mile
				when 'mm'   then distance.mm
				when 'cm'   then distance.cm
				when 'km'   then distance.km
				else distance.inch
			end
			#now convert the distance in inches to desired output
			distance_result = case result
				when 'in'   then distance_inches.to_inch
				when 'ft'   then distance_inches.to_feet
				when 'yard' then distance_inches.to_yard
				when 'mile' then distance_inches.to_mile
				when 'mm'   then distance_inches.to_mm
				when 'cm'   then distance_inches.to_cm
				when 'km'   then distance_inches.to_km
				else distance.to_inch
			end
			return distance_result
		end

		
		
		
		
		def onKeyUp(key, repeat, flags, view)
			#get the current key pressed
			@toggle_keys = @@im_values[@@c_key_ids[key].to_s] = false if(@@c_key_ids.has_key?(key))
			#toggle settings if toggle keys pressed
			case @@c_key_ids[key].to_s
				when "collision"
					@@d_settings["collision"] = (@@d_settings["collision"]) ? false : true
					Sketchup::set_status_text("Collision "+((@@d_settings["collision"]) ? "enabled" : "disabled"),SB_PROMPT)		
				when "complexcl"
					@@d_settings["collision"] = (@@d_settings["complexcl"]) ? @@d_settings["collision"] : true
					@@d_settings["complexcl"] = (@@d_settings["complexcl"]) ? false : true
					Sketchup::set_status_text("Complex Collision "+((@@d_settings["complexcl"]) ? "enabled" : "disabled"),SB_PROMPT)		
				when "cameraupr"
					@@d_settings["cameraupr"] = (@@d_settings["cameraupr"]) ? false : true
					Sketchup::set_status_text("Camera Upright "+((@@d_settings["cameraupr"]) ? "enabled" : "disabled"),SB_PROMPT)		
				when "restrictv"
					@@d_settings["restrictv"] = (@@d_settings["restrictv"]) ? false : true
					Sketchup::set_status_text("Restrict Vertical Movement "+((@@d_settings["restrictv"]) ? "enabled" : "disabled"),SB_PROMPT)		
				when "camerahgt"
					@@d_settings["camerahgt"] = (@@d_settings["camerahgt"]) ? false : true
					Sketchup::set_status_text("Eye Height "+((@@d_settings["camerahgt"]) ? "enabled" : "disabled"),SB_PROMPT)		
			end #case
		end #def
		
		def onKeyDown(key, repeat, flags, view)
			#get the current key pressed
			@toggle_keys = @@im_values[@@c_key_ids[key].to_s] = true if(@@c_key_ids.has_key?(key))

			if(!@animation_active)
				@animation_active = true
				@animation_start  = true
				view.animation = self
				view.dynamic = true
			end #if
		
		end #def
		
		def onLButtonDown(flags, x, y, view)
			#store the original mouse coordinates
			@toggle_mouse = true
			if(!@mouse_stored || !(@mousex || @mousey))
				@mouse_stored = true
				@mousex = x
				@mousey = y
				@currentx = x
				@currenty = y
			else
				#restore the previous mouse condition !!!!BRILLIANT!!!!
				@mousex = x - (@currentx - @mousex)
				@mousey = y - (@currenty - @mousey)
				@currentx = x
				@currenty = y
			end #if
			if(!@animation_active)
				@animation_active = true
				@animation_start  = true
				view.animation = self
				view.dynamic = true
			end #if
		end #def
		
		def onLButtonUp(flags, x, y, view)
			@toggle_mouse = false
		end #def
		
		def onMouseMove(flags, x, y, view)
			if @toggle_mouse
				@currentx = x
				@currenty = y
				if x <= 3 || y <= 3
					@animation_active = false
				end #if
			end
		end #def
		
		
		def nextFrame(view)
			if ((!@toggle_keys && (@speed_move<=0.5 && @speed_fov<=0.1 && @speed_roll<=0.1)) && (!@@d_settings["camerahgt"] && @speed_height<=0.1) && (!@toggle_mouse && @speed_look<=1) || !@animation_active)
				#dead camera so end the animation
				@mouse_stored     = false
				@animation_active = false
				view.animation = nil
				#and restore the view conditions
				view.dynamic = false
				Sketchup.vcb_label = "Measurements"
				Sketchup.vcb_value = ""
				return false
			else
				#animation is live so calculate movement and then refresh the view
				time_delay = 0			
				if(@animation_start)
					time = @@q_refresh_rate
					render_time = 0
					@animation_degrade = false
					@animation_start   = false
				else
					render_time = view.last_refresh_time
					time = render_time
				end #if
				#adjust the time if slower than refresh rate
				if(time < @@q_refresh_rate)
					time_delay = @@q_refresh_rate - time
					time = @@q_refresh_rate
				end #if

				if((render_time > (@@q_refresh_rate*0.75)) && !@animation_degrade)
					@animation_degrade = true					
					self.setRenderOptions(false,false,false,@@q_degrade_opts)
				end #if
				
				power = time/@@q_refresh_rate
				
				old_fov = view.field_of_view
				view.field_of_view = 1
				
				#initial change of FOV of camera
				if ((@@im_values["shrink"] || @@im_values["grow"]) || @speed_fov > 0.1)
					#calculate differential vector direction
					deltav = 0
					dev    = 0
					deltav=-1 if @@im_values["shrink"]
					deltav= 1 if @@im_values["grow"]
					#apply this to the vector
					diff_vec = Geom::Vector3d.new(1,0,0)
					diff_vec.length = deltav
					
					#modify acc based on current fov speed and proximity to maximums
					acc = 50 * (Math.sin(@rad_90*((100-@speed_fov)/100))**power) * (Math.sin(@rad_90*((old_fov-1)/55.5))**power)
					diff_vec.length = acc * time * time if (deltav != 0)
					
					#dev = 1 if (diff_vec.valid? && @p_vector_fov.valid? && (@p_vector_fov.samedirection? diff_vec))
					#p_speed_fov = @speed_fov * ((1-(Math.cos(dev/2)/10))**(power))
					dev = (@p_vector_fov[0]*diff_vec[0] < 0) ? 0.913 : 0.9
					p_speed_fov = @speed_fov * (dev**power)

					@p_vector_fov.length = (p_speed_fov * time) if @p_vector_fov.valid?
					@p_vector_fov += diff_vec
					@speed_fov = @p_vector_fov.length / time if @p_vector_fov.valid?
					if @speed_fov >= 0.5
						Sketchup.vcb_label = "Field of Vision (FOV)"
						current_fov = old_fov + @p_vector_fov[0]
						current_fov = (current_fov * 10).round.to_f / 10
						Sketchup.vcb_value = current_fov
					end #if

					old_fov += @p_vector_fov[0]
				end #if
				
				#initial change of pitch and pan of camera
				camera_up = Geom::Vector3d.new(0,0,1)
				if(!@@d_settings["cameraupr"])
					camera_up = view.camera.up
					if((@@im_values["clock"] || @@im_values["anti"]) || @speed_roll > 0.1)
						deltar = 0
						dev = 0
						deltar=-1 if @@im_values["anti"]
						deltar= 1 if @@im_values["clock"]
						diff_vec = Geom::Vector3d.new(1,0,0)
						diff_vec.length = deltar
						
						acc = 100 * (Math.sin(@rad_90*((100-@speed_roll)/100))**power)
						
						diff_vec.length = diff_vec.length * acc * time * time if diff_vec.valid?
						
						#dev = @rad_90 if (diff_vec.valid? && @p_vector_roll.valid? && (@p_vector_roll.samedirection? diff_vec))
						#p_speed_roll = @speed_roll * (1-(Math.cos(dev/2)/10))
						dev = (@p_vector_roll[0]*diff_vec[0] < 0) ? 0.929 : 0.9
						p_speed_roll = @speed_roll * (dev**power)

						@p_vector_roll.length = (p_speed_roll * time) if @p_vector_roll.valid?
						@p_vector_roll += diff_vec
						@speed_roll = @p_vector_roll.length / time if @p_vector_roll.valid?

						roll = Geom::Transformation.rotation view.camera.eye, (view.camera.eye.vector_to view.camera.target), @p_vector_roll[0] * 0.02

						camera_up.transform! roll
						view.camera.set(view.camera.eye, view.camera.target, camera_up)
					end #if
				# else
					# #camera_up = view.camera.up
					# camera_yaxis = view.camera.yaxis
					# camera_yaxis[2] = 0
					# camera_up_2d = view.camera.xaxis.cross(camera_yaxis)
					# up_vector = Geom::Vector3d.new(0,0,1)
					# x_vector = up_vector.cross(view.camera.eye.vector_to view.camera.target)
					# #x_vector = Geom::Vector3d.new(x_vector[0].abs,x_vector[1].abs,x_vector[2].abs)
					
					# angle = camera_up_2d.angle_between up_vector
					
					# if(angle > (1.degrees/2))
						# angle90 = x_vector.angle_between camera_up_2d
						# angle360 = (angle90 > @rad_90) ? Math::PI + (Math::PI-angle) : angle
						# puts angle90.to_s
						
						# deltar = 0
						# dev = 0
						# deltar=-1 if angle360 >  Math::PI
						# deltar= 1 if angle360 <= Math::PI
						# diff_vec = Geom::Vector3d.new(1,0,0)
						# diff_vec.length = deltar
						
						# acc = 100 * Math.sin(@rad_90*((100-@speed_roll)/100)) * Math.sin(@rad_90*(angle/2))
						
						# diff_vec.length = diff_vec.length * acc * time * time if diff_vec.valid?
						
						# dev = (@p_vector_roll[0]*diff_vec[0] < 0) ? 0.929 : 0.9
						# p_speed_roll = @speed_roll * (dev**power)

						# @p_vector_roll.length = (p_speed_roll * time) if @p_vector_roll.valid?
						# @p_vector_roll += diff_vec
						# @speed_roll = @p_vector_roll.length / time if @p_vector_roll.valid?
						
						
						# roll = Geom::Transformation.rotation view.camera.eye, (view.camera.eye.vector_to view.camera.target), @p_vector_roll[0] * 0.02

						# camera_up_2d.transform! roll
						# view.camera.set(view.camera.eye, view.camera.target, camera_up_2d)
						# camera_up = camera_up_2d
					# else
						# @speed_roll = 0
					# end #if
				end #if
				
				if (@toggle_mouse || @speed_look > 1)
					cam2d = Geom::Point3d.new(@mousex,@mousey,0)														 #fictional position of the camera
					diff_vec = Geom::Vector3d.new((@currentx-@mousex),(@currenty-@mousey),0) #direction of the force
					dist_mult = (diff_vec.valid?) ? diff_vec.length : 0
					dist_mult = 1270 if dist_mult > 1270
					dist_mult = 0 if (dist_mult < 5) && @@d_settings["infinite_rot"]
					dist_mult = Math.sin(@rad_90*(dist_mult/1270))
					#get the acceleration as a product of the distance to the destination
					acc = 20000 * dist_mult
					diff_vec.length = acc * time * time if diff_vec.valid?
					
					#modify the previous_vector
					dev = (diff_vec.valid?) ? (@p_vector_look.angle_between diff_vec) : 0
					@speed_look = @speed_look * (1-(Math.cos(dev/2)/4)) * dist_mult
					@p_vector_look.length = @speed_look * time if @p_vector_look.valid?
					#calculate the new vector by combining the old vector with the differential vector
					@p_vector_look += diff_vec
					#calculate the new mousex and mousey points
					cam2d = cam2d + @p_vector_look

					xrot = @p_vector_look[0]
					yrot = @p_vector_look[1]
					
					elev = view.camera.direction.angle_between Geom::Vector3d.new(0,0,1)
					yrot = yrot * Math.sin((elev*(@rad_reduceby2)) + (@rad_1))
					
					cameray_vector = view.camera.eye.vector_to view.camera.target
					camerax_vector = cameray_vector.cross(view.camera.up)
					
					yrotation = Geom::Transformation.rotation view.camera.eye, camerax_vector, (-yrot * @max_yrotation)
					xrotation = Geom::Transformation.rotation view.camera.eye, view.camera.up, (-xrot * @max_rotation)
					rotation = yrotation * xrotation
					
					target = view.camera.target
					target.transform!(rotation)
					camera_up.transform!(rotation) if(!@@d_settings["cameraupr"])
					view.camera.set(view.camera.eye, target, Geom::Vector3d.new(0,0,1))
					
					#calculate the previous_vector
					@speed_look = @p_vector_look.length / time
					if (!@@d_settings["infinite_rot"] || (@@d_settings["infinite_rot"] && !@toggle_mouse))
						@mousex = cam2d[0]
						@mousey = cam2d[1]
					end #if
				end #if
				
				#camera normal vector
				eye = view.camera.eye
				camera_normal = view.camera.target - eye
				camera_normal[2] = 0 if (@@d_settings["camerahgt"])
				camera_normal.normalize!
				#camera cross vector
				camera_cross = camera_normal.cross(Geom::Vector3d.new(0,0,1))
				camera_cross.normalize!
				#camera cross perp vector
				camera_cross_perp = (@@d_settings["restrictv"]) ? Geom::Vector3d.new(0,0,1) : camera_cross.cross(camera_normal)
				camera_cross_perp.normalize!
				
				#work out delta values of the differential vector
				deltax = 0 
				deltay = 0
				deltaz = 0
				deltax= 1 if @@im_values['forward']
				deltax=-1 if @@im_values['backward']
				deltay= 1 if @@im_values['right']
				deltay=-1 if @@im_values['left']
				if(!@@d_settings["camerahgt"])
					deltaz= 1 if @@im_values['up']
					deltaz=-1 if @@im_values['down']
				end #if
				#apply the delta values to a camera normal and cross vector
				camera_cross.length = deltay
				camera_normal.length = deltax
				if(!@@d_settings["camerahgt"])
					camera_cross_perp.length = deltaz
					diff_vec = camera_normal + camera_cross + camera_cross_perp
				else
					diff_vec = camera_normal + camera_cross
				end #if
				diff_vec.normalize!
				
				#define the acceleration by deviation and friction (@max speed)
				dev = (diff_vec.valid?) ? (@p_vector_move.angle_between diff_vec) : 0	
				acc = @acc_constant * Math.sin(@rad_90*(1-(Math.cos(dev/2)/2))) * Math.sin(@rad_90*((@max_speed-@speed_move)/@max_speed))	
				#apply friction to the previous velocity and also correct this for the refresh rate
				@speed_move = @speed_move * (@@d_settings["friction_scalar"]**(power))-((@@d_settings["friction_scalar"]*(1-((Math.sin(dev))/4)))**(power))
					
				#convert unit vector into a vector as a product of differential velocity
				diff_vec.length = acc * time * time if diff_vec.valid?
				#convert the previous vector to its new velocity
				@p_vector_move.length = @speed_move * time if (@p_vector_move.valid?)
				#work out new vector for velocity by combining previous vector with differential vector
				@p_vector_move += diff_vec

				
				#work out the current camera height by shooting a ray in the direction 0,0,-1
				#grab the distance
				if (@@d_settings["camerahgt"])
					#if eye height is 0 then initial distance = desired distance
					target_start = eye + @max_height_vector
					target_end = target_start + Geom::Vector3d.new(0,0,-1)
					#max_height_object = @collision_group.entities.add_line target_start, target_end
					ray = [eye, @max_height_vector]
					result = Sketchup.active_model.raytest(ray)
					#@collision_group.entities.erase_entities max_height_object
					
					if result
						intersect_point = result[0]
						intersect_path  = result[1]
						if !(@collision_group == intersect_path[0])
							height = eye.distance intersect_point
							#first modify the eye height based on the key presses
							@eye_height += 4 if @@im_values['up'] && (@eye_height > 0)
							@eye_height -= 4 if @@im_values['down'] && (@eye_height > 0)
							@p_vector_height.length = 0 if (@eye_height == 0 && @p_vector_height.valid?)
							@eye_height = height if (@eye_height == 0)
							
							#get the difference between the desired eye height and
							#the current eye height
							deltah = 0
							deltah = @eye_height - height if @eye_height
							diff_vec = Geom::Vector3d.new(0,0,1)
							diff_vec.length = deltah
							#get an acceleration value based
							acc = (@acc_constant) * Math.sin(@rad_90*((@max_speed-@speed_height)/@max_speed))

							dist_mult = deltah.abs
							dist_mult = 3.5 if (dist_mult > 3.5)
							dist_mult = (Math.cos(@rad_90*((3.5-dist_mult)/3.5))**(power))
							
							acc = acc * dist_mult
							#apply friction to the previous velocity and also correct this for the refresh rate
							@speed_height = @speed_height * dist_mult * (0.97**power)
							
							#convert unit vector into a vector as a product of differential velocity
							diff_vec.length = acc * time * time if diff_vec.valid?
							#convert the previous vector to its new velocity
							@p_vector_height.length = @speed_height * time if (@p_vector_height.valid?)
							#work out new vector for velocity by combining previous vector with differential vector
							@p_vector_height += diff_vec
							
							distance = @p_vector_height.length
							if distance != 0
								distance_mm = self.distanceConverter(height,'in','mm')
								distance_mm = (distance_mm * 10).round.to_f / 10
								Sketchup.vcb_label = "Eye Height (mm)"
								Sketchup.vcb_value = distance_mm
							end #if
							@speed_height = distance/time
							
							movement = Geom::Transformation.translation(@p_vector_height)
							eye = view.camera.eye.transform(movement)
							target = view.camera.target.transform(movement)
							view.camera.set(eye, target, camera_up)
						else
							#work out delta values of the differential vector
							deltaz = 0
							deltaz= 1 if @@im_values['up']
							deltaz=-1 if @@im_values['down']
							#apply the delta values to a camera normal and cross vector
							camera_cross_perp.length = deltaz
							#get the differential vector
							diff_vec = camera_cross_perp
							#define the acceleration by deviation and friction (@max speed)
							dev = (diff_vec.valid?) ? (@p_vector_move.angle_between diff_vec) : 0	
							acc = @acc_constant * Math.sin(@rad_90*(1-(Math.cos(dev/2)/2))) * Math.sin(@rad_90*((@max_speed-@speed_move)/@max_speed))	
							#apply friction to the previous velocity and also correct this for the refresh rate
							#convert unit vector into a vector as a product of differential velocity
							diff_vec.length = acc * time * time if diff_vec.valid?
							#work out new vector for velocity by combining previous vector with differential vector
							@p_vector_move += diff_vec
							@eye_height = 0
						end #if
					end #if
				else
					@eye_height = 0
				end #if

				#work out collision detection by drawing a vector of set distance in the direction of travel (@p_vector_move)
				#rather than affect the diff_vec as this doesn't convey the momentum
				#so first get a vector from the camera eye along @p_vector_move and check for face intersections
				deflection_vector = false
				if (@@d_settings["collision"])
					sane_vector = @p_vector_move.clone
					sane_vector.normalize!
					sane_vector.length = sane_vector.length * 200
					target_start = eye + sane_vector
					target_end = target_start + sane_vector
					
					#max_intersect_object = @collision_group.entities.add_line target_start, target_end
					
					ray = [eye, @p_vector_move]
					result = Sketchup.active_model.raytest(ray)
					#@collision_group.entities.erase_entities max_intersect_object
					
					if result
						intersect_point = result[0]
						intersect_path  = result[1]
						#if true, we've got a valid intersection
						if !(@collision_group == intersect_path[0])
							intersect_entity = intersect_path[(intersect_path.length-1)]
							#get the distance between the points
							distance = eye.distance intersect_point
							#get the angle between the normal and the vector of movement and multiply the
							#distance max to suit this.
							if intersect_entity.is_a?(Sketchup::Face)
								normal = intersect_entity.normal
								for i in (0..(intersect_path.length-2)) do
									normal = normal.transform(intersect_path[i].transformation)
								end
								incidence_angle = normal.angle_between @p_vector_move
								distance = distance * Math.cos(incidence_angle)
								distance = distance.abs
								
								normal_cross = normal.cross(@p_vector_move)
								deflection_vector = normal_cross.cross(normal)
								
								if (distance <= @collision_buffer)
									deflection_vector.length = @p_vector_move.length * Math.sin(incidence_angle)
								else
									deflection_vector = false
								end
							end #if
						end
					end
				end #if
				
				view.field_of_view = old_fov
				movement_vector = (deflection_vector) ? deflection_vector : @p_vector_move
				#work out the displacement relative to the camera normal and cross vectors?
				movement = Geom::Transformation.translation(movement_vector)
				eye = view.camera.eye.transform(movement)
				target = view.camera.target.transform(movement)
				view.camera.set(eye, target, camera_up)
				#get the previous distance travelled				
				distance = movement_vector.length
				@speed_move = distance/time #converts to inches on divide, dirty cock!!!
				speed_kh = self.distanceConverter(@speed_move,'in','km')
				speed_kh = speed_kh*3600
				speed_kh = (speed_kh * 10).round.to_f / 10
				if @speed_height <= 0.1 && @speed_fov <= 0.5
					Sketchup.vcb_label = "Speed (km/h)"
					Sketchup.vcb_value = speed_kh
				end #if

				view.show_frame time_delay
			end	
			
		end #def
		
	end
#end

@@float_tool = false

	if( not file_loaded?(__FILE__))

		#write the defaults on first run
		if Sketchup.read_default("Floating Camera","INIT","fail") == "fail"
			Sketchup.write_default("Floating Camera","INIT",false)
			Sketchup.write_default("Floating Camera","KCOLLISION",VK_INSERT)
			Sketchup.write_default("Floating Camera","KRESTRICTV",VK_DELETE)
			Sketchup.write_default("Floating Camera","KCAMERAHGT",191)
			Sketchup.write_default("Floating Camera","KCAMERAUPR",222)
			Sketchup.write_default("Floating Camera","KUP",VK_SHIFT)
			Sketchup.write_default("Floating Camera","KDOWN",VK_CONTROL)
			Sketchup.write_default("Floating Camera","KSHRINK",VK_HOME)
			Sketchup.write_default("Floating Camera","KGROW",  VK_END)
			Sketchup.write_default("Floating Camera","KCLOCK", 102)
			Sketchup.write_default("Floating Camera","KANTI",  100)
			
			Sketchup.write_default("Floating Camera","DMROTATION",540)
			Sketchup.write_default("Floating Camera","DMSPEEDKH",20)
			Sketchup.write_default("Floating Camera","DFRICTION",10)
			Sketchup.write_default("Floating Camera","DIROTATION",180)
			Sketchup.write_default("Floating Camera","DINFROTAT",true)
			Sketchup.write_default("Floating Camera","DCOLLISION",false)
			Sketchup.write_default("Floating Camera","DRESTRICTV",false)
			Sketchup.write_default("Floating Camera","DCAMERAHGT",false)
			Sketchup.write_default("Floating Camera","DCAMERAUPR",true)
		end #if
		

		
		@@float_tool = false

		submenu = UI.menu("Camera").add_submenu("Floating Camera")
		cmd = UI::Command.new("Floating Camera") {
			@@float_tool = CControl::CameraFloatTool.new
			Sketchup.active_model.select_tool @@float_tool
		}
		submenu.add_item(cmd)
		
		submenu.add_separator
		
		cmd = UI::Command.new("Change Controls") {
			@@float_tool = CControl::CameraFloatTool.new
			@@float_tool.openControlsPanel()
		}
		submenu.add_item(cmd)
		
		cmd = UI::Command.new("Motion Settings") {
			@@float_tool = CControl::CameraFloatTool.new
			@@float_tool.openSettingsPanel()
		}
		submenu.add_item(cmd)

		tb = UI::Toolbar.new("Floating Camera")
		cmd = UI::Command.new("Floating Camera") {
			@@float_tool = CControl::CameraFloatTool.new
			Sketchup.active_model.select_tool @@float_tool
		}
		# plugin_filepath = __FILE__
		# puts plugin_filepath
		# plugin_path = File.dirname(__FILE__)
		# puts plugin_path
		# resource_path = Sketchup.get_resource_path('helpcontent')
		# puts resource_path
		# plugins_folder = Sketchup.find_support_file('Plugins')
		# puts plugins_folder
		# model_path = Sketchup.active_model.path
		# puts model_path
		# plugin_path2 = plugin_path+"/../../../../../../"
		# expanded_path = File.expand_path(plugin_path2)
		# puts expanded_path
		# plugin_path = File.dirname(__FILE__)
		# root_dir_entries = Dir.entries(plugin_path+"/../../../../../../")
		# puts root_dir_entries
		#filename_path = File.dirname ( $FILENAME )
		#puts filename_path
		#filename_path = File.dirname ( $LOAD_PATH )
		#puts filename_path
		
		#probably errors because there's no support file
		plugin_path = File.dirname( __FILE__ )
		cameraimg_location = plugin_path + "/floating_camera/instructor/instructor_files/floating_camera_camera.png"
		cameraimg_small_location = plugin_path + "/floating_camera/instructor/instructor_files/floating_camera_camera_small.png"
		cmd.large_icon = cameraimg_location
		cmd.small_icon = cameraimg_small_location
		cmd.tooltip = "Esc to exit"
		cmd.menu_text = "Floating Camera"
		tb.add_item(cmd)
		
		cmd = UI::Command.new("Change Controls") { 
			@@float_tool = CControl::CameraFloatTool.new if !@@float_tool
			@@float_tool.openControlsPanel()
		}
		controlsimg_location = plugin_path + "/floating_camera/instructor/instructor_files/floating_camera_controls.png"
		controlsimg_small_location = plugin_path + "/floating_camera/instructor/instructor_files/floating_camera_controls_small.png"
		cmd.large_icon = controlsimg_location
		cmd.small_icon = controlsimg_small_location
		cmd.tooltip = "Set the controls for the camera"
		cmd.menu_text = "Camera Controls"
		tb.add_item(cmd)
		
		cmd = UI::Command.new("Motion Settings") { 
			@@float_tool = CControl::CameraFloatTool.new if !@@float_tool
			@@float_tool.openSettingsPanel()
		}
		settingsimg_location = plugin_path + "/floating_camera/instructor/instructor_files/floating_camera_settings.png"
		settingsimg_small_location = plugin_path + "/floating_camera/instructor/instructor_files/floating_camera_settings_small.png"
		cmd.large_icon = settingsimg_location
		cmd.small_icon = settingsimg_small_location
		cmd.tooltip = "Adjust the camera behaviour"
		cmd.menu_text = "Camera Settings"
		tb.add_item(cmd)
		
		tb.show

	  file_loaded(__FILE__)
	end

end

