#  ==========================================================================
#  load_instructor.rb
#  --------------------------------------------------------------------------
#
#  Mixin Module : LoadInstructor
#
#  by Niall Campbell, 2011-03(-17)
#
#  version :  1.0.1
# v1.0.2 >= v2014 compatible TIG 20150606
#
#  ==========================================================================
#
#  CHANGELOG:
#  ------------------
#  = v 1.0.1 (2011-03-17)
#  	-Added folder in temp path creation if folders in path don't exist
#  	-Added proper temp file deletion (fix to Directory Not Empty Error)
#
#
#  DEFINITION OF USE:
#  ------------------
# #NB:		This definition of use describes the basic use of
#					load_instructor.rb. For more advanced uses, see
# 				the Users Guide available at:
# 				http://forums.sketchucation.com/viewtopic.php?f=180&t=35806
#
# #Appendix: -$LOAD_PATH refers to any of sketchup's load paths, like:
# 						/Tools/ or /Plugins/
#
#  (1)		Create a 'mixin' subdir under one of the $LOAD_PATH paths.
#					The 'Plugins' folder is fine.
#
#  (2)		Copy this file 'load_instructor.rb' to that 'mixin' dir.
#
#  (3)		Create an 'instructor' subdir beneath your Tool plugin's
#					subdir (ie. [your_scripts_files]/instructor), and place your 
#					instructor files beneath this 
#
# #NB:		Your instructor folder should have a file 'index.html'. This
#					is the file that will be displayed in the instructor window
#
#  (4)		Add the lines to require() and include() THIS mixin module
#					to your tool class definition:
#
#					require('mixin/load_instructor.rb')
#					include( Mixin::LoadInstructor )
#
#
#  (5)	Call the lins_setup('your_scripts_files') method from inside
#					your Tool class initialize() method, like the example below:
#
# #NB:		calling lins_setup with only one attribute like:
# 				lins_setup('your_scripts_files') will use the
# 				the default methods for constructing paths to the temp
# 				directory and your instructor folder as follows:
#
#					instructor_folder = [any $LOAD_PATH]/your_scripts_files/instructor
#					if PC
#						temporary_directory = ENV["TEMP"]/your_scripts_files/
#					if MAC
#						dont worry about the temp directory as everything can
#						be accessed by a relative path so there's no need to copy
#
# Example:
#
#
# #NB: This example assumes the following file structure
#
# [any $LOAD_PATH]/mixin/load_instructor.rb
# [any $LOAD_PATH]/your_script.rb
# [any $LOAD_PATH]/your_scripts_files/some_image.png
# [any $LOAD_PATH]/your_scripts_files/etc..
# [any $LOAD_PATH]/your_scripts_files/instructor
# [any $LOAD_PATH]/your_scripts_files/instructor/index.html
# [any $LOAD_PATH]/your_scripts_files/instructor/etc..
#
# module MyModule
#
# 	class MyCoolScript
# 		# this line of code just includes the load_instructor.rb
# 		# file from the path '[any $LOAD_PATH]/mixin/load_instructor.rb'
# 		require('mixin/load_instructor.rb')
#
# 		# to add the load_instructor.rb methods to your script
# 		# you need to include the following line
# 		include( Mixin::LoadInstructor )
# 		# now all the load_instructor methods are available to
# 		# your class

#
#    	def initialize
#      	# Run the Load Instructor setup by using the following
#				# line of code
#      	lins_setup('your_scripts_files')
#
#
#				# You get and adjust the values of the instance variables:
#				# @instructor_folder & @temp_folder here if need be, like:
# 			#
#				# Returning the folder variables
#				 instructor_folder = @instructor_folder.dup
# 			# temp_folder = @temp_folder.dup
# 			# #NB: .dup copies the variable so when you modify
# 			# instructor_folder or temp_folder you won't modify
# 			# @instructor_folder or @temp_folder
# 			#
#				# Setting the folder variables
#				# @instructor_folder = instructor_folder
#				# @temp_folder = temp_folder
#
#				#..Other tool init code goes here...
#
#    	end
#
# 		#...the rest of your script...
#
# 	end #class MyCoolScript
#
# end #module MyModule
#
#  Now you should have a consistent Instructor Directory across drives.
#
#	 You can return the instructor temp folder name by calling:
#  lins_return_temp_instructor_folder_name() from inside your
#  tools code, OR by calling: temp_instructor_folder() from
#  outside.
#  (Returns false if no temp folder has yet been created.
#  Returns a relative path from the temp folder if the temp
#  folder has been created, like: 'your_scripts_files/instructor'
#
#
#  Finally, you can get the relative path from the Sketchup 
#  'helpcontent/tool' folder to the tool's instructor folder
#  (whether it was copied to the temp folder or not) by calling:
#  lins_return_relative_path() from inside your tool's code, OR
#  by calling:  mytool.relative_instructor_folder() from ouside.

module Mixin

	module LoadInstructor
		attr_accessor :instructor_folder, :temp_folder
    # CLASS VARIABLES
    @@relative_instructor_path = ''
    @@temp_instructor_folder = false


    public
    
    def relative_instructor_path
    	lins_return_relative_path()
      return @@relative_instructor_path
    end
    
    def temp_instructor_folder
    	lins_return_temp_instructor_folder_name()
      return @@temp_instructor_folder
    end

 
    private
		
    # lins_setup( instructor_folder=false, temp_folder=false )
    #
    # Sets up and inits some instance variables used for
    # the tool's Instructor help system.
    #
    # ** Must be called by the Tool's initialize() method. **
    #
		def lins_setup(script_folder, instructor_folder=false, temp_folder=false)
      #
			if instructor_folder
				@instructor_folder = instructor_folder
			else
				@instructor_folder = lins_get_default_instructor_folder(script_folder)
			end
			if (not RUBY_PLATFORM =~ /(mswin|mingw)/i) # on a Mac
				@temp_folder = @instructor_folder
			elsif temp_folder
				@temp_folder = temp_folder
			else
				@temp_folder = lins_get_default_temp_folder(script_folder)
			end
			#
		end #def lins_setup()
		
    # lins_get_default_instructor_folder()
    #
    # Constructs a default instructor folder by using
    # script_folder as a parent and instructor as its
    # child (ie. '[script_folder]/instructor')
    #
		def lins_get_default_instructor_folder(script_folder)
			instructor_folder = ''
			$LOAD_PATH.each {|load_path|
				check_path = File.join(load_path, script_folder, 'instructor')
				if check_path != '(eval)' && File.exist?(check_path)
					instructor_folder = File.join(load_path, script_folder, 'instructor')
					break
				end
			}
			if instructor_folder.empty?
				file_path = File.join(script_folder, 'instructor')
				raise(ScriptError,"#{self.class.name}: Instructor folder error. Could not find #{file_path} beneath any of the $LOAD_PATH's.")
			else
				return instructor_folder
			end #if
		end #def
		
    # lins_get_default_temp_folder()
    #
    # Returns the default (user) temp folder only
    # for PC. Mac would use ENV["TMPDIR"] but its
    # not needed as mac volumes can be accessed
    # relatively
    #
		def lins_get_default_temp_folder(script_folder)
			temp_folder = File.join(File.expand_path(ENV["TEMP"]), script_folder)
		end #def
		
	
		# lins_get_image_type( file )
    #
    # adapted from:
    # http://boonedocks.net/mike/archives/162-Determining-Image-File-Types-in-Ruby.html
		# Checks for jpg, png, gif, tif, tga & bmp and variations in their data formats.
    #
		def lins_get_image_type(file)
		
			filestart = IO.read(file,30)
			case filestart
				when /^BM/
					return 'bmp'
				when /^GIF8/
					return 'gif'
				when /^MM\x00\*(\x00){3}/
					return 'tif'
				when /^II\*\x00\x08(\x00){3}\x15/
					return 'tif'
				when /^#{"‰PNG"}/
					return 'png'
				when /^#{"ÿØÿà"}/
					return 'jpg'
				#when /^\xff\xd8\xff\xe0/
				#	return 'jpg'
				#when /^\xff\xd8\xff\xe1(.*){2}Exif/
				#	return 'jpg'
				when /^\x00\x00\x0A\x00/
					return 'tga'
				when /^\x00\x00\x02\x00/
					return 'tga'
				else
					fileext = File.extname(file).downcase
					case fileext
						when /^\.(?i)(tga|vda|icb|vst)/
							return 'tga'
						when /^\.(?i)(tif|tiff)/
							return 'tif'
					end #case
					puts "unknown: "+IO.read(file,20).dump
					return 'unknown'
			end
		end

		# lins_check_validity( filename )
    #
		def lins_check_validity(filename)
      # grab the file type:
      #  'file','directory','characterSpecial','blockSpecial',
      #  'fifo','link','socket','unknown'
			filetype = File.ftype(filename)
			#
			
			case filetype
				when "file"
					if (!File.readable?(filename))
						puts "file is not readable: "+filename
						return "unreadable"
					end #if
					if (!File.size?(filename))
						puts "file exists but has 0 size: "+filename
						return "invalidsize"
					end #if
					image_type = lins_get_image_type(filename)
					if(image_type == 'unknown')
						puts "file exists but is unknown (assume text file): "+filename
						return "unknowntype"
					end #if
					return "imgfile"
				when "directory"
					return "directory"
			end #case
		end #def
		
		
		def lins_delete_temp_files(temp_dir)
			entries = Dir.entries(temp_dir)
			entries.delete(".")
			entries.delete("..")
			entries.each{|entry|
				entry_absolute_path = File.join(temp_dir,entry)
				file_validity = lins_check_validity(entry_absolute_path)
				case file_validity
					when "directory"
						#recurse through the child directories and delete the files
						lins_delete_temp_files(entry_absolute_path)
						Dir.delete(entry_absolute_path)
					else
						File.delete(entry_absolute_path)
				end
			}
		end #def

		
    # lins_copy_instructor_files( temp_folder, dir, basedir )
    #
		# call like:
    # lins_copy_instructor_files([temp_dir],[base_dir_of_instructor_folder],[base_dir_of_instructor_folder])
    # dir changes on recursion through the child directories and basedir 
    # stays constant to maintain the folder/file structure in the temp folder
    #
		def lins_copy_instructor_files(temp_folder, dir, basedir)
			puts "running copy"
			entries = Dir.entries(dir)
			if (dir != basedir)
				temp_path = dir.dup
				temp_path.slice!(basedir)
				temp_folder_abs_path = temp_folder+temp_path
				Dir.delete(temp_folder_abs_path) if(File.exist?(temp_folder_abs_path))
				Dir.mkdir(temp_folder_abs_path, 0700)
			else
				temp_folder_abs_path = temp_folder
			end #if
			entries.delete(".")
			entries.delete("..")
			
			#create a group and a texture writer object for copying imgfiles
			entries.each {|entry|
				entry_absolute_path = File.join(dir, entry)
				file_validity = lins_check_validity(entry_absolute_path)
				case file_validity
					when "directory"
						#recurse through the child directories and copy the files
						lins_copy_instructor_files(temp_folder, entry_absolute_path, basedir)
					when "imgfile"
						grp = Sketchup.active_model.entities.add_group
						tw  = Sketchup.create_texture_writer
						#so create a material name
						filename = File.basename(entry_absolute_path)
						fileext  = File.extname(filename)
						filebase = File.basename(entry_absolute_path, fileext)
						matname = "instructor_"+filebase + rand(1000).to_s
						
						#load the image into a material
						mat = Sketchup.active_model.materials.add(matname)
						mat.texture = entry_absolute_path
						mat.texture.size = [mat.texture.image_height,mat.texture.image_width]
						
						#apply the material to a group and export to temp directory
						grp.material = mat
						tw.load(grp)
						result = tw.write(grp, File.join(temp_folder_abs_path,entry))
						
						case result
							when 0
								puts "file written: "+temp_folder_abs_path+"/"+filename
							when 1; puts "file failed (invalid tiff): "+temp_folder_abs_path+"/"+filename
							when 2; puts "file failed (unknown): "+temp_folder_abs_path+"/"+filename
						end #case
						
						#delete the material
						grp.material = nil
						Sketchup.active_model.materials.remove mat
						grp.erase! if grp.valid?
					when "unknowntype"
						source = File.open(entry_absolute_path)
						target = File.open(File.join(temp_folder_abs_path,entry), "w")
						
						target.write( source.read(64) ) while not source.eof?
						
						source.close
						target.close
				end #case
			}
		end #def
		
    # lins_copy_instructor()
    #
		# call after lins_setup
		# Checks first if running on a mac (@instructor_folder != @temp_folder)
		# and then checks if the temp and instructor folders are located on the
		# same volume (instructor_path_array[0] != temp_path_array[0]). If not
		# on a mac and not on the same drive, then files can't be accessed
		# by a relative path, so we copy the instructor_folder to the temp dir
		# 
    #
		def lins_copy_instructor
			puts "copying instructor"
			@instructor_folder = File.expand_path(@instructor_folder)
			@temp_folder       = File.expand_path(@temp_folder)		
			#on mac there's no need to copy, just to generate a correct relative path
			#as other volumes can be accessed through the /volumes/ folder
			#so @instructor_folder == @temp_folder only on mac
			if @instructor_folder != @temp_folder
				#check if paths are on the same drive in windows, if they are
				#there's no need to copy as they can be accessed through relative paths
				instructor_path_array = @instructor_folder.split("/")
				temp_path_array = @temp_folder.split("/")
				if instructor_path_array[0] != temp_path_array[0]
					#create the base instructor folder in the temp folder
					#directory and copy all the instructor files to this
					#folder
					check_array = Array.new
					0.upto(temp_path_array.length-2) {|i| 
						check_array.push(temp_path_array[i])
						check_path = check_array * "/"
						Dir.mkdir(check_path, 0700) if (not File.exist?(check_path))
					}
					lins_delete_temp_files(@temp_folder) if(File.exist?(@temp_folder))
					Dir.delete(@temp_folder) if(File.exist?(@temp_folder))
					Dir.mkdir(@temp_folder, 0700)
					lins_copy_instructor_files(@temp_folder, @instructor_folder, @instructor_folder)
				else
					@temp_folder = @instructor_folder
					puts "on same drive"
				end
			else
				puts "on a mac"
			end #if
		end #def

    # lins_return_temp_instructor_folder_name()
    #
    # Returns the relative path from ENV["TEMP"] if instructor
    # was copied to the temp folder, returns false otherwise
    #		
		def lins_return_temp_instructor_folder_name
			return @temp_folder if @temp_folder == @instructor_folder
			instructor_path_array = @instructor_folder.split("/")
			temp_path_array = @temp_folder.split("/")
			if (@instructor_folder != @temp_folder) && (instructor_path_array[0] != temp_path_array[0])
				temp_instructor_folder_name = @temp_folder.dup
				temp_instructor_folder_name.slice!(File.expand_path( ENV["TEMP"] ))
				temp_instructor_folder_name[1,temp_instructor_folder_name.length]
				@@temp_instructor_folder = temp_instructor_folder_name
			else
				false
			end
		end #def
		
    # lins_return_relative_path()
    #
    # Constructs the smallest possible relative path to
    # the instructor folder regardless of whether it was
    # copied to the temp folder or not
    #		
		def lins_return_relative_path
			tool_path = Sketchup.get_resource_path( 'helpcontent/tool' )
			temp_path = @temp_folder
			#need to reverse loop through tool path by the separator until
			#we get a match starting at the first line of the temp path
			relativity_index = 0
			common_directory_path = temp_path
			tool_path_array = File.expand_path(tool_path).split('/')
			0.upto(tool_path_array.length-1) {|i|
				common_directory = tool_path_array[0, tool_path_array.length - i]
				if RUBY_PLATFORM =~ /(mswin|mingw)/i # on a PC
					common_directory_path = common_directory.join('/')
				else
					common_directory_path = '/'+common_directory.join('/')
				end
				if common_directory_path == temp_path[0,common_directory_path.length]
					break
				end
				relativity_index = relativity_index.next
			}
			
			temp_path = temp_path[common_directory_path.length, temp_path.length]
			path_to_common = "..#{File::SEPARATOR}" * relativity_index
			@@relative_instructor_path = File.join( path_to_common, temp_path)
			@@relative_instructor_path = @@relative_instructor_path.gsub(File::SEPARATOR + File::SEPARATOR, File::SEPARATOR)
			
			@@relative_instructor_path = @instructor_folder if Sketchup.version.to_i > 13
			
		end #def


		def getInstructorContentDirectory
			unless @@temp_instructor_folder
				lins_copy_instructor
				lins_return_temp_instructor_folder_name
			end
			lins_return_relative_path
		end #def		
	
	end #module
	
end #module